from robot_face import RobotFace
import time

if __name__ == '__main__':
    aimee = RobotFace(robot_name='aimee', voice_id='Justin')
    nathan = RobotFace(robot_name='nathan', voice_id='Matthew')

    # send_message(request)
    while True:
        message = input("who will we make talk?: ")
        if message == 'nathan':
            nathan.update_face({'pupil_scale': .4})
            nathan.express({'AU1':.5, 'AU4l': 0.5}, 1000)
            nathan.say('It is nice to meet you, I am a robot face')
            
        if message == 'aimee':
            nathan.release_gaze()
            aimee.say('It is nice to meet you, I am a robot face')
            nathan.stop_speech()
        


# from TTS.api import TTS

# tts = TTS("tts_models/multilingual/multi-dataset/xtts_v2")

# # generate speech by cloning a voice using default settings
# tts.tts_to_file(text="It took me quite a long time to develop a voice, and now that I have it I'm not going to be silent.",
#                 file_path="output2.wav",
#                 speaker_wav="./male.wav",
#                 language="en")

from transformers import Wav2Vec2Processor, Wav2Vec2ForCTC 
from datasets import load_dataset
import torch
import soundfile as sf
import numpy as np

# load model and processor
processor = Wav2Vec2Processor.from_pretrained("vitouphy/wav2vec2-xls-r-300m-timit-phoneme")
model = Wav2Vec2ForCTC.from_pretrained("vitouphy/wav2vec2-xls-r-300m-timit-phoneme")

# Read and process the input
audio_input, sample_rate = sf.read("output.wav")

def chunk_with_overlap(arr, chunk_size, overlap):
    return [arr[i:i + chunk_size] for i in range(0, len(arr) - chunk_size + 1, overlap)]

audio_input = chunk_with_overlap(audio_input, int(sample_rate * .1), int(sample_rate * .05)) # chunk into 150 ms segments
audio_input = np.array(audio_input[:-1])
print(audio_input.shape)

inputs = processor(audio_input, sampling_rate=16_000, return_tensors="pt", padding=True)

print(inputs.input_values.shape)

with torch.no_grad():
    logits = model(inputs.input_values, attention_mask=inputs.attention_mask).logits

# Decode id into string
predicted_ids = torch.argmax(logits, axis=-1)      
predicted_sentences = processor.batch_decode(predicted_ids)

print(predicted_sentences)


#!/usr/bin/env python
"""The setup script."""

from setuptools import find_packages, setup

with open("README.md", encoding="utf-8") as readme_file:
    readme = readme_file.read()

with open("HISTORY.md", encoding="utf-8") as history_file:
    history = history_file.read()

# NOTE: Update pinned_reqs whenever install_requires or extras_require changes.
install_requires = [
    # numpy>=1.17.0 is when default_rng becomes available;
    # scikit-learn 1.1.0 requires numpy 1.17.3+
    "numpy>=1.17.3",
    "numpy_groupies>=0.9.16",  # Supports Python 3.7 and up.
    "numba>=0.51.0",
    "pandas>=1.0.0",
    "sortedcontainers>=2.0.0",  # Primarily used in SlidingBoundariesArchive.
    "scikit-learn>=1.1.0",  # Primarily used in CVTArchive.
    "scipy>=1.7.0",  # Primarily used in CVTArchive.
    "threadpoolctl>=3.0.0",
]

extras_require = {
    "dev": [
        "pip>=20.3",
        "pylint",
        "yapf",
        "pre-commit",

        # Testing
        "pytest==7.0.1",
        "pytest-cov==3.0.0",
        "pytest-benchmark==3.4.1",
        "pytest-xdist==2.5.0",

        # Documentation
        "myst-nb==0.17.1",
        "Sphinx==4.5.0",
        "sphinx-autobuild==2021.3.14",
        "sphinx-autodoc-typehints==1.18.2",
        "sphinx-codeautolink==0.12.1",
        "sphinx-copybutton==0.3.1",
        "sphinx-jinja2-compat==0.2.0",
        "sphinx-material==0.0.32",
        "sphinx-prompt==1.5.0",
        "sphinx-tabs==3.3.1",
        "sphinx-toolbox==3.1.0",
        "sphinxcontrib-applehelp==1.0.4",
        "sphinxcontrib-devhelp==1.0.2",
        "sphinxcontrib-htmlhelp==2.0.1",
        "sphinxcontrib-jsmath==1.0.1",
        "sphinxcontrib-qthelp==1.0.3",
        "sphinxcontrib-serializinghtml==1.1.5",

        # Distribution
        "bump2version==1.0.1",
        "wheel==0.40.0",
        "twine==4.0.2",
        "check-wheel-contents==0.4.0",
    ],
}

setup(
    author="Interaction Lab",
    author_email="dennler@usc.edu",
    classifiers=[
        "Development Status :: 4 - Beta",
        "Intended Audience :: Developers",
        "Intended Audience :: Education",
        "Intended Audience :: Science/Research",
        "License :: OSI Approved :: MIT License",
        "Natural Language :: English",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: 3.11",
        "Programming Language :: Python :: 3.12",
        "Topic :: Software Development :: User Interfaces",
        "Topic :: Communications :: Chat",
        "Topic :: Scientific/Engineering :: Human Machine Interfaces",
    ],
    description=
    "An interface for embodied speech to text.",
    install_requires=install_requires,
    extras_require=extras_require,
    license="MIT license",
    long_description=readme + "\n\n" + history,
    long_description_content_type="text/markdown",
    include_package_data=True,
    keywords="lips",
    name="pylips",
    packages=find_packages(include=["pylips", "pylips.*"]),
    python_requires=">=3.8.0",
    test_suite="tests",
    # url="pylips.readthedocs.io",
    version="0.0.2",
    zip_safe=False,
)


import socketio
import boto3
from contextlib import closing
import json
import pickle
import pygame
import time
import os

class RobotFace:
    def __init__(self,
                 robot_name: str = 'default',
                 server_ip: str = 'http://localhost:8000',
                 tts_method: str = 'polly',
                 voice_id: str = 'Justin'):
        '''
            The main class for the Web-Enabled Facial Text-to-Speech (WEFacTTS) project.
            :param: robot_name - the identity of the robot that should be speaking
            :param: server_ip - the location of the server running the flask application.
        '''
        self.name = robot_name
        self.voice_id = voice_id

        self.io = socketio.Client()
        self.io.connect(server_ip)

        pygame.mixer.init()
        self.channel = 0

        tts_methods = ['polly']
        if tts_method not in tts_method:
            raise Exception(f'parameter tts_method must be one of {tts_methods}')
        
        # TODO: implement other TTS things, but for now we will use polly
        if tts_method == 'polly':
            self.tts = boto3.client('polly')

    def say(self, content):
        '''
            The main method for the RobotFace class. This method will take in a string of text and
            convert it to speech using the AWS Polly service.
            :param: content - the text that the robot should speak
        '''

        response = self.tts.synthesize_speech(
            TextType='ssml',
            Text=f"<speak>{content}</speak>",
            OutputFormat='mp3',
            VoiceId=self.voice_id
        )

        # Get the audio stream
        audio_stream = response["AudioStream"]

        # Play the audio stream
        with open(f"{self.name}_output.mp3", "wb") as file:
            file.write(audio_stream.read())
        
        
        
        # Synthesize speech with viseme output
        response = self.tts.synthesize_speech(
            TextType='ssml',
            Text=f"<speak>{content}</speak>",
            OutputFormat='json',
            SpeechMarkTypes=['viseme'],
            VoiceId=self.voice_id
        )

        # Process the response
        if 'AudioStream' in response:
            # Do something with the audio stream if needed
            with closing(response["AudioStream"]) as stream:
                        data = stream.read().decode('utf-8')
                        xSheet = data.split('\n')
                        xSheet = [json.loads(line) for line in xSheet if line != '']
        
        VIS2IPA = {"p": "BILABIAL",
                "f": "LABIODENTAL",
                "T": "INTERDENTAL",
                "s": "DENTAL_ALVEOLAR",
                "t": "DENTAL_ALVEOLAR",
                "S": "POSTALVEOLAR",
                "r": "POSTALVEOLAR",
                "k": "VELAR_GLOTTAL",
                "i": "CLOSE_FRONT_VOWEL",
                "u": "CLOSE_BACK_VOWEL",
                "@": "MID_CENTRAL_VOWEL",
                "a": "OPEN_FRONT_VOWEL",
                "e": "OPEN_FRONT_VOWEL",
                "E": "OPEN_FRONT_VOWEL",
                "o": "OPEN_BACK_VOWEL",
                "O": "OPEN_BACK_VOWEL",
                "sil": "IDLE"}

        times = [x['time'] / 1000. for x in xSheet]
        visemes = [VIS2IPA[x['value']] for x in xSheet]

        request = { 
                'name': self.name,
                'action_type': 'say',
                'visemes': visemes,
                'times': times,
             }
        

        #load sound
        while pygame.mixer.Channel(self.channel).get_busy():
            self.channel += 1
             
        sound = pygame.mixer.Sound(f"{self.name}_output.mp3")
        # pygame.mixer.Channel(self.channel).music.load(f"{self.name}_output.mp3")

        
        #play sound and face
        self.io.emit('face_control', request)
        pygame.mixer.Channel(self.channel).play(sound)


    def say_file(self, filename):
        if not os.path.exists(filename):
             raise Exception(f'phrase {filename} does not exist')
        times, visemes = pickle.load(open(filename, 'rb'))

        request = { 
                'name': self.name,
                'action_type': 'say',
                'visemes': visemes,
                'times': times,
             }

    def stop_speech(self):
        request = { 
                'name': self.name,
                'action_type': 'stop_speech',
             }
        
        self.io.emit('face_control', request)
        pygame.mixer.Channel(self.channel).stop()
         
        
    def look(self, x, y, z, time):
        '''
        Looks to a location in the face's reference frame.
        x,y,z in mm
        time in ms
        '''
        request = {
                'name': self.name,
                'action_type': 'look',
                'location': [x,y,z],
                'time': time
            }
        self.io.emit('face_control', request)

    def release_gaze(self):
        '''
        allows robot to move eyes idly
        '''
        request = {
            'name': self.name,
            'action_type': 'release_gaze',
        }
        self.io.emit('face_control', request)

    def express(self, aus, time):
        request = {
            'name': self.name,
            'action_type': 'express',
            'aus': aus,
            'time': time
        }
        self.io.emit('face_control', request)

    def update_face(self, config):
        request = {
            'name': self.name,
            'action_type': 'update_face',
            'configuration': config
        }
        self.io.emit('face_control', request)
        time.sleep(.3)


         
